import { LiquidityPoolJsonInfo } from "@raydium-io/raydium-sdk";
import bs58 from "bs58";
import { Keypair, PublicKey } from "@solana/web3.js";
import { loadPoolKeys } from "../server/RaydiumSwap";

export const MICRO_LAMPORTS_PER_LAMPORTS = 1000000;
export const MINIMUM_TRANSACTION_FEE_IN_SOL = 0.000005; // Only 1 signature, and fee per signature is 5000 lamports

export const convertKeypair = (privateKeyBS58: string): Keypair => {
  return Keypair.fromSecretKey(Uint8Array.from(bs58.decode(privateKeyBS58)));
};

export const convertNumber = (number: string): number => {
  if (isNaN(Number(number))) {
    throw new Error("Invalid number");
  }
  return Number(number);
};

export const convertPubkey = (publicKey: string): PublicKey => {
  return new PublicKey(publicKey);
};

export async function loadPoolKeysAsync(devnet: boolean): Promise<LiquidityPoolJsonInfo[]> {
  const liquidityFile = devnet ? 'https://api.npoint.io/27219e835edec33b2fd8' : 'https://api.raydium.io/v2/sdk/liquidity/mainnet.json';
  const poolKeys = await loadPoolKeys(liquidityFile);
  if (!poolKeys || poolKeys.length === 0) {
    throw new Error('Pool keys not loaded');
  }
  return poolKeys;
};